/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <msp430x16x.h>
#include "smallib.h"
#include "config.h"
#include "funk.h"
#include "routing.h"
#include "net.h"
#include "driver_flash.h"
#include "driver_timer.h"
#include "driver_system.h"
#include "driver_sht11.h"
#include "command.h"
#include "helpers.h"
#include "file.h"
#include "boot.h"

#include "user_apps.h"
#include "timer.h"

#define NL	(10)	// New line
#define CR	(13)	// Carriage return

void readLine(char* buffer, int maxLength) {
	int j = 0;
	unsigned char tmp = 0;

	while (j == 0 || (tmp != CR && tmp != NL && j < maxLength))
	{
		if (mEventFlags.Bits.char_in)
		{
			tmp = sl_getchar();
			buffer[j++] = tmp;
			if (tmp != CR && tmp != NL) sl_putchar(buffer[j-1]);
			else j--;
		}
	}
	buffer[j] = 0;
}

void read(char* buffer, int len) {
	int j = 0;
	while (j < len)	{
		if (mEventFlags.Bits.char_in) {
			buffer[j++] = sl_getchar();
		}
	}
}

void showMenu(void) {
	sl_printf("============================ M E N U ================ v1.0.0 ========");
	sl_printf("\n");
	sl_printf("\n");
	sl_printf("a - Displays the current UID of this node.");
	sl_printf("\n");
	sl_printf("b - Repeatadly sends a fixed packet to another node and measures the time.");
	sl_printf("\n");
	sl_printf("c - Constant send mode (sending preambles).");
	sl_printf("\n");
	sl_printf("d - Measure temperature and humidity of a node.");
	sl_printf("\n");
	sl_printf("e - Send a network command.");
	sl_printf("\n");
	sl_printf("f - Constant receive mode.");
	sl_printf("\n");
	sl_printf("g - PWD mode of CC1100.");
	sl_printf("\n");
	sl_printf("l - Enter low power mode 3.");
	sl_printf("\n");
	sl_printf("m - Displays this menu.");
	sl_printf("\n");
	sl_printf("p - Adjust the output power.");
	sl_printf("\n");
	sl_printf("r - Displays RSSI value of last received packet.");
	sl_printf("\n");
	sl_printf("s - Displays the sequence number buffer.");
	sl_printf("\n");
	sl_printf("t - Displays the current routing table.");
	sl_printf("\n");
	sl_printf("u - Set the UID of this node.");
	sl_printf("\n");
	sl_printf("v - Change velocity settings for carrera.");
	sl_printf("\n");
	sl_printf("x - Send a message to another node (with echo).");
	sl_printf("\n");
	sl_printf("k - Close the logfile, if one is open.");
	sl_printf("\n");
	sl_printf("\n");
	sl_printf("ESC - Resets the module.");
	sl_printf("\n");
	sl_printf("\n");
	sl_printf("=====================================================================");
	sl_printf("\n");
	sl_printf("\n");
}

void comDisplayCurrentAddress(void) {
	sl_printf("Current node address: %u\n", uid);
}

void comSendBurst(void) {
	int i, c;
	int ack_count = 0;
	char buffer[58];
	uint8_t address = 0;
	uint32_t start, duration;

	sl_printf("Packet count: ");
	readLine(buffer, 4);
	sl_printf("\n");

	c = sl_atoi(buffer);
	if (c == 0 || c < 1 || c > 9999) {
		sl_printf("Input Error");
		sl_printf("\n");
		return;
	}

	sl_printf("Destination: ");
	readLine(buffer, 3);
	sl_printf("\n");

	address = (uint8_t) sl_atoi(buffer);
	if (address == 0) {
		sl_printf("Input Error");
		sl_printf("\n");
		return;
	}

	// Content of buffer doesn't matter, finish string
	buffer[57] = 0;

	start = *TICKS();
	for (i = 0; i < c; i++)
	{
		csend(address, 0x03, (uint8_t*) buffer, PACKET_LENGTH-4); // Length always 58 bytes (timing)
		if (rflags.LL_ACK) ack_count++;
	}
	duration = *TICKS() - start;
	sl_printf("Duration: %lu \n", duration);;
	sl_printf("#ACKs received: %u\n", ack_count);
}

void comSetConstantSendMode(void) {
	WOR_WAKEUP();
	rflags.RX_REQ = false;
	spiStrobe(CC1100_STX);
}

void comSetConstantReceiveMode(void) {
	WOR_WAKEUP();
	rflags.RX_REQ = false;
	spiWriteReg(CC1100_MCSM2, 0x07);
	spiStrobe(CC1100_SRX);
}

void comSetCC1100SpwdMode(void) {
	WOR_WAKEUP();
	rflags.RX_REQ = false;
	spiStrobe(CC1100_SIDLE);
	spiStrobe(CC1100_SPWD);
}


void comMeasureTempAndHumi(void) {
	uint8_t address;
	char buffer[8];

	sl_printf("Select node: ");
	readLine(buffer, 3);
	sl_printf("\n");

	address = (uint8_t) sl_atoi(buffer);
	if (address == 0) {
		sl_printf("Input Error\n");
		return;
	}

	if (address == uid) {
#ifdef INCLUDE_SHT11
		// Local
		int h;
		float t;
		sht11_update_data();
		t = sht11_data.temp_val.f;
		h = (int) sht11_data.humi_val.f;
		float2string(buffer, t, 2, 3, 1);
		sl_printf("Temperature: %s°C\n", buffer);
		sl_printf("Humidity: %u%%RH\n", h);
#endif
	} else {
		// Remote
		buffer[0] = uid;// For reply
		buffer[1] = 1;	// Temp&Humi
		bool res = net_send(129, address, (uint8_t*)buffer);
		if (!res) {
			sl_printf("TX buffer full\n");
		}
	}
}


void comAdjustOutputPower(bool local) {
	char buffer[3];
	uint8_t index = 0;

	sl_printf("CC1100 output power selection [0-11]:\n");
	sl_printf("0 ^= -52 dBm\n");
	sl_printf("1 ^= -30 dBm\n");
	sl_printf("2 ^= -20 dBm\n");
	sl_printf("3 ^= -15 dBm\n");
	sl_printf("4 ^= -10 dBm\n");
	sl_printf("5 ^= -5 dBm\n");
	sl_printf("6 ^= -1 dBm\n");
	sl_printf("7 ^= 0 dBm\n");
	sl_printf("8 ^= +5 dBm\n");
	sl_printf("9 ^= +7 dBm\n");
	sl_printf("10 ^= +9 dBm\n");
	sl_printf("11 ^= +10 dBm\n");
	sl_printf("Current PATABLE Index: %u\n", paTableIndex);
	sl_printf("Enter new PATABLE Index: ");
	readLine(buffer, 2);
	sl_printf("\n");

	index = (uint8_t) sl_atoi(buffer);
	if (index > 11) {
		sl_printf("Input Error");
		sl_printf("\n");
		return;
	}

	if (local) {
		bool res = setOutputPower(index);
		if (res) {
			sl_printf("PATABLE Index successfully set!\n");
		}
	} else {
		buffer[0] = index;
		sendLayer1Command(COMMAND_OUTPUT_POWER,(uint8_t*) buffer);
	}
}

void comSendNetworkCommand(void) {
	char buffer[3];
	uint8_t index = 0;

	sl_printf("Command index [1-2]: \n\n");
	sl_printf("1 - Reset all other nodes\n");
	sl_printf("2 - Set output power on all other nodes\n");
	sl_printf("Enter command index: ");
	readLine(buffer, 2);
	sl_printf("\n");

	index = (uint8_t) sl_atoi(buffer);
	if (index > 2) {
		sl_printf("Input Error\n");
		return;
	}

	switch (index) {
		case COMMAND_RESET:
			sendLayer1Command(COMMAND_RESET, (uint8_t*)buffer);
			break;
		case COMMAND_OUTPUT_POWER:
			comAdjustOutputPower(false);
			break;
		default:
			break;
	}
}

void comDisplayRSSI(void) {
	sl_printf("RSSI: %ddBm\n", getLastRSSI());
}

void comDisplaySeqNumBuffer(void) {
	sl_printf("SEQ-BUFFER: \n");
	printSeqNumBuffer();
	sl_printf("\n");
}

void comDisplayRoutingTable(void) {
	sl_printf("Routing table: \n");
	printRoutingTable();
	sl_printf("\n");
}

void comSetCurrentAddress(void) {
	uint8_t address;
	char buffer[4];

	sl_printf("New UID: ");
	readLine(buffer, 3);
	sl_printf("\n");

	address = (uint8_t) sl_atoi(buffer);
	if (address == 0) {
		sl_printf("Input Error\n");
		return;
	}

	if (address != uid) {
		setUid(address);
		writeNodeId(address);
	}
}

void comSendMessage(void) {
	char buffer[52];

	sl_printf("Destination: ");
	readLine(buffer, 3);
	sl_printf("\n");

	uint8_t nodeId = (uint8_t) sl_atoi(buffer);
	if (nodeId == 0 ) {
		sl_printf("Input Error\n");
		return;
	}

	buffer[0] = uid; // Store our UID for echo reply

	sl_printf("Message: ");
	readLine(buffer+1, 49);
	sl_printf("\n");

	bool res = net_send(128, nodeId, (uint8_t*)buffer);
	if (!res) {
		sl_printf("TX buffer full\n");
	}
}

void com_handle_net_paket(void) {
	typedef struct paket{
		uint8_t type;
		uint8_t dest;
		uint8_t data[50];
	}paket;
	paket p;
	char *buffer = (char*) &p;
	read(buffer, 52);
	if (net_send(p.type, p.dest, (uint8_t*) &(p.data))){
		sl_printf("OK\n");
	}
	else {
		sl_printf("FAIL\n");
	}
}

#ifdef INCLUDE_SDCARD
#ifdef INCLUDE_FAT16
void comCloseLog(void)
{
	if (log_file_open)
	{
		file_write(&log_file,"</sensor_values>\n</xml>\n",24);
		log_file_open=0;
		file_closefile(&log_file);
		uart_init(115200,0);
	}
}
#endif
#endif

void comSetCarreraSpeed() {
	char buffer[4];
	uint8_t value = 0;

	sl_printf("Speed [1-100]:\n");
	sl_printf("Enter new speed: ");
	readLine(buffer, 3);
	sl_printf("\n");

	value = (uint8_t) sl_atoi(buffer);
	if (value > 100 || value < 1) {
		sl_printf("Input Error");
		sl_printf("\n");
		return;
	}

	set_speed(value);
}


//void ChangeCarreaSettings()  expects a String that contains 3 Parameters
//First: Minspeed contains two character: e.g.  05
//Second: Maxspeed contains two character: e.g.  14
//Third: Breaktime contains four character: e.g.  1250
//Finally the String should look like this: 05141250\0 with exact 9 characters

void ChangeCarreaSettings() {

	char bufMin[3];
	char bufMax[3];
	char bufBreaktime[5];
	char bufXThreshold[5];
	char buffer[13];

	readLine(buffer, 12);

	bufMin[0]=buffer[0];
	bufMin[1]=buffer[1];
	bufMin[2]=0;
	MINSPEED=(uint16_t) sl_atoi(bufMin);

	bufMax[0]=buffer[2];
	bufMax[1]=buffer[3];
	bufMax[2]=0;
	MAXSPEED=(uint16_t) sl_atoi(bufMax);

	bufBreaktime[0]=buffer[4];
	bufBreaktime[1]=buffer[5];
	bufBreaktime[2]=buffer[6];
	bufBreaktime[3]=buffer[7];
	bufBreaktime[4]=0;
	BREAKTIME=(uint16_t) sl_atoi(bufBreaktime);

	bufXThreshold[0]=buffer[8];
	bufXThreshold[1]=buffer[9];
	bufXThreshold[2]=buffer[10];
	bufXThreshold[3]=buffer[11];
	bufXThreshold[4]=0;
	XACCTHRESHOLD = (uint16_t) sl_atoi(bufXThreshold);
}


void old_com_exec(char c) {
	switch (c) {
		case '#':
			com_handle_net_paket();
			break;
		case 'a':
			comDisplayCurrentAddress();
			break;
		case 'b':
			comSendBurst();
			break;
		case 'c':
			comSetCarreraSpeed();
			//comSetConstantSendMode();
			break;
		case 'd':
			comMeasureTempAndHumi();
			break;
		case 'e':
			comSendNetworkCommand();
			break;
		case 'f':
			comSetConstantReceiveMode();
			break;
		case 'g':
			comSetCC1100SpwdMode();
			break;
		case 'l':
			GoPowerDown();
			break;
		case 'm':
			showMenu();
			break;
		case 'p':
			comAdjustOutputPower(true);
			break;
		case 'Q': //QUIT Carrera
#ifdef SCHIENE
			stop_car();
			sl_printf("quit\n");
#endif
			break;
		case 'S': //START Carrera
#ifdef SCHIENE
			start_car();
			sl_printf("start\n");
#endif
			break;
		case 't':
			comDisplayRoutingTable();
			break;
		case 'u':
			comSetCurrentAddress();
			break;
		case 'v':
			ChangeCarreaSettings();
			break;
		case 'x':
			comSendMessage();
			break;
		case CR:
		case NL:
			// Ignore at this point!
			break;
#ifdef INCLUDE_SDCARD
#ifdef INCLUDE_FAT16
		case 'k':
			comCloseLog();
			break;
#endif
#endif
		default:
			sl_printf("Command '%c' not recognized\n",c);
	}
}


// ---------------------------------------------------------------- //
// Simple serial protocol demo (for interaction with WSN Simulator) //
// ---------------------------------------------------------------- //

// State machine values
#define STATE_INIT 				(0)
#define STATE_COM_ID_1 			(1)
#define STATE_COM_ID_2 			(2)
#define STATE_COM_ID_3			(3)
#define STATE_COM_READ 			(4)
#define STATE_COM_TYPE_READ 	(5)
#define STATE_COM_LENGTH_READ	(6)

// Command identifier - command starts with: !#!
#define COM_ID_1	(33)
#define COM_ID_2	(35)
#define COM_ID_3	(33)

// Available commands
#define COM_1	(1)	// Query sensor data

// Command type
#define COM_REQ (1)	// Request
#define COM_RES (2) // Response

static char state = STATE_INIT;
static char commandID = 0;
static char commandType = 0;
static uint8_t commandDataIdx = 0;
static uint8_t commandDataMax = 0;
static char commandData[10];

void com_reset(void) {
	state = STATE_INIT;
    commandID = 0;
    commandType = 0;
    commandDataIdx = 0;
    commandDataMax = 0;
}

void com_measure_sht11(uint8_t* buffer)
{
#ifdef INCLUDE_SHT11
	uint8_t pos = 0;
	sht11_update_data();
	buffer[pos++] = sizeof(float);
	sl_memcpy(buffer + pos, &sht11_data.temp_val.f, sizeof(float));
	pos += sizeof(float);
	buffer[pos++] = sizeof(uint16_t);
	int h = (int) sht11_data.humi_val.f;
	sl_memcpy(buffer + pos, &h, sizeof(uint16_t));
#endif
}

void com_1_respond(uint8_t length, uint8_t* buffer)
{
	uint8_t i;
	sl_putchar((char)COM_ID_1);
	sl_putchar((char)COM_ID_2);
	sl_putchar((char)COM_ID_3);
    sl_putchar((char)COM_RES);
    sl_putchar((char)COM_1);
    sl_putchar((char)length);
    for (i = 0; i < length; i++) {
    	sl_putchar((char)buffer[i]);
    }
}

void com_1_remote_respond(uint8_t dest)
{
	// Buffer containing remote measured data
	uint8_t buffer[5 + sizeof(float) + sizeof(uint16_t)];
	buffer[0] = COM_1;
	buffer[1] = sizeof(buffer) - 2;
	buffer[2] = uid;
	com_measure_sht11(buffer + 3);
	net_send(141, dest, buffer);
}

void com_1(void) {
	// Buffer containing local measured data
	uint8_t buffer[3 + sizeof(float) + sizeof(uint16_t)];
	// Local request
	if (commandData[0] == uid) {
		buffer[0] = uid;
		com_measure_sht11(buffer + 1);
		com_1_respond(sizeof(buffer), buffer);
	}
	// Remote request, send request to destination
	else {
		buffer[0] = COM_1;
		buffer[1] = uid;
		net_send(140, commandData[0], buffer);
	}
}

void serial_request(uint8_t source, uint8_t* data)
{
	switch (data[0])
	{
		case COM_1:
			com_1_remote_respond(data[1]);
			break;
	}
}

void serial_response(uint8_t source, uint8_t* data)
{
	switch (data[0])
	{
		case COM_1:
			com_1_respond(data[1], data + 2);
			break;
	}
}

void com_execute(void) {
    switch (commandType) {
        case COM_RES:
            // Not supported
            break;
        case COM_REQ:
        	if (commandID == COM_1) {
                com_1();
            }
            break;
    }
    com_reset();
}

void execCommand(char c) {
	switch (state) {
        case STATE_INIT:
            if (c == COM_ID_1) state = STATE_COM_ID_1;
            else old_com_exec(c);
            break;
        case STATE_COM_ID_1:
            if (c == COM_ID_2) state = STATE_COM_ID_2;
            break;
        case STATE_COM_ID_2:
            if (c == COM_ID_3) state = STATE_COM_ID_3;
            break;
        case STATE_COM_ID_3:
            if (c == COM_REQ) {
                commandType = COM_REQ;
                state = STATE_COM_TYPE_READ;
            } else if (c == COM_RES) {
                commandType = COM_RES;
                state = STATE_COM_TYPE_READ;
            } else {
            	com_reset();
            }
            break;
        case STATE_COM_TYPE_READ:
            commandID = c;
            state = STATE_COM_READ;
            break;
        case STATE_COM_READ:
        	commandDataMax = c;
        	state = STATE_COM_LENGTH_READ;
        	break;
        case STATE_COM_LENGTH_READ:
            commandData[commandDataIdx++] = c;
            if (commandDataIdx == commandDataMax) {
            	com_execute();
            }
            break;
    }
}
