/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"
#ifdef INCLUDE_FAT16
#include "fat.h"
#include <stdlib.h>

#define MBR_SECTOR			0 		/* Sectornummer des MBR*/
#define VBR_OFS 			0x1C6 	/* Adresse der VBR Adresse im MBR*/

fat_io_callback fat_read_cb;
fat_io_callback fat_write_cb;
fat_flush_callback fat_flush_cb;
volume_info vi;

int fat_init_media(fat_io_callback rcb,fat_io_callback wcb,fat_flush_callback fcb)
	{
		uint8_t fat_copies;	
		uint8_t *sector_buffer;

		fat_read_cb = rcb;
		fat_write_cb = wcb;
		fat_flush_cb = fcb;
			
		/* MBR einleseb	*/
		sector_buffer = fat_read_cb(MBR_SECTOR); /*MBR einlesen*/

		/* Sectornummer des Volume Boot Record ist an Adresse VBR_ADDR im MBR zu finden */
		vi.vbr_sector = *(uint16*) (sector_buffer + VBR_OFS);

		/* VBR einlesen */
		sector_buffer = fat_read_cb(vi.vbr_sector);
		
		vi.bytes_per_Sector = (uint16_t) ((sector_buffer[12]<<8) + sector_buffer[11]);
		vi.sec_per_cluster =  sector_buffer[13];
		
		/*Check for right sector size*/
		if (vi.bytes_per_Sector!=512)
		{
			 return FAT_INCOMPATIBLE_SECTOR_SIZE;
		}
		
		vi.fat_sector = (uint16_t) ((sector_buffer[15]<<8) + sector_buffer[14]) + vi.vbr_sector;
		
		/* Rootdirposition ist Fatgröße in Sektoren * Fatanzahl + volumeBootRecordAdress*/
		fat_copies = sector_buffer[16];
		vi.fat_size = (uint16_t) ((sector_buffer[23]<<8) + sector_buffer[22]);
		vi.root_dir_sector = (vi.fat_sector + (fat_copies * vi.fat_size));
		
		/* Datenanfang (Cluster 0) = Rootdirpos + max direinträge * direintraglänge (32Byte) / Sektorgröße (512Byte)*/  		
		vi.data_sector = (((uint16_t) ((sector_buffer[18]<<8) + sector_buffer[17]) * 32) / 512) + vi.root_dir_sector;
				
		return 0;
	}

uint8_t *fat_read_root_dir_sector(uint16_t sector)
	{
		/* Sector liegt nicht in Rootdir*/
		if ((sector+vi.root_dir_sector)>= vi.data_sector) return NULL;
		return fat_read_cb(vi.root_dir_sector+sector);
	}

int fat_get_next_cluster(uint16_t cluster)
	{	 
		if (cluster > 0xfff7) 
		{
			return FAT_END_OF_CHAIN;
		}
		uint8_t *buffer;
		uint16_t tmp;
		/*Sektor des Clusters im Fat ausrechnen*/
		uint16_t offset= (cluster*2)/512;
		/*Entsprechenden Fat-Sektor lesen*/
		buffer=fat_read_cb(vi.fat_sector+offset);
		/*Offset in Fat-Sektor*/
		offset=(cluster*2)%512;
		tmp= *(uint16*) (buffer+offset);
		/*übers Ende hinaus gelesen*/
		if (tmp > 0xfff7) 
			{
				tmp = FAT_END_OF_CHAIN;
			}
		return tmp;
	}	
	
uint16_t fat_get_first_free_cluster(void)
{
	register int i;
	uint8_t *buffer;
	uint32_t sector;
	uint16_t cluster;
	uint16_t offset;
	for (i = 0; i < vi.fat_size * 512 / 2; i++)
		{
			sector = (i*2) / 512 + vi.fat_sector;
			buffer = fat_read_cb(sector);
			offset = (i*2) % 256;
			cluster = *(uint16*) (buffer+offset);
			if (cluster==0) return i;
		}
	return 0;
}

uint16_t fat_add_cluster(uint16_t cluster)
{
	register int i;
	if (cluster == 1) return 0;
	signed long newcluster=fat_get_first_free_cluster();
	signed long towrite=newcluster;
	/*Disk Full*/
	if (newcluster==0) return 0;

	// first round: Append new cluster
	// second round: Set cluster end mark
	for (i=0;i<2;i++)
	{
		if (cluster)
		{
			fat_write_cluster(cluster,towrite);
		}
		cluster= newcluster;
		towrite=FAT_END_OF_CHAIN;
	}
	return newcluster;
}

int fat_unlink_chain(uint16_t cluster)
{	
	uint32_t next;
	do
	{
		next = fat_get_next_cluster(cluster);
		fat_write_cluster(cluster,0);
	} while (next!= FAT_END_OF_CHAIN);
	return 0;
}

int fat_write_cluster(uint16_t cluster,uint16_t link)
{
	uint32_t sector;
	register uint16_t offset;
	uint8_t *buffer;
	sector = ((cluster*2)/512) + vi.fat_sector;
	buffer = fat_read_cb(sector);
	offset=((cluster)*2)%512;
	*(uint16*) (buffer+offset) = link;
	fat_write_cb(sector);
	return 0;
}
#endif
