/* 
 
 Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */


/* ----------------------------------------------------------------------
 * /src/hal_includes/driver_cc1100.h
 * 
 * Header for the platform dependend Part of the cc1100 driver. User 
 * normally doesn't call any of these functions.
 * If you plan to implement micro-mesh to another platform, you have to
 * fully implement this header-file. 
 * -----------------------------------------------------------------------
 */
#ifndef DRIVER_CC1100_H_
#define DRIVER_CC1100_H_

#include "type.h"

/* Definitions to support burst/single access */
#define CC1100_WRITE_BURST	(0x40) /* Offset for burst write. */
#define CC1100_READ_SINGLE	(0x80) /* Offset for read single byte. */
#define CC1100_READ_BURST	(0xC0) /* Offset for read burst. */
#define NOBYTE				(0x00) /* No command, for reading. */

/*-------------------------------------------------------------------------------------------------------
 *	void cc1100driver_reset(void)
 *
 *	DESCRIPTION:
 *		Reset the CC1100 and wait for it to be ready. Afterwards CC1100 is in IDLE mode, so
 *		setWorMode() must be called.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_reset(void);


/*-------------------------------------------------------------------------------------------------------
 *  void cc1100driver_spiinittrx(void)
 *
 *  DESCRIPTION:
 *		This function puts the cc1100 into spi mode. You have to call this bevore every spi transaction.
 *  
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_spiinittrx(void);


/*-------------------------------------------------------------------------------------------------------
 *  uint8_t cc1100driver_trxspi(uint8_t)
 *
 *  DESCRIPTION:
 *		This function sends and receives a byte via SPI.
 *
 *  ARGUMENTS:
 *      uint8_t
 *          Data to be send via SPI.
 *
 *  RETURN VALUE:
 *      UINT8
 *          Return value from CC1100 (SO Pin).
 *---------------------------------------------------------------------------------------------------
 */
extern uint8_t drivercc1100_trxspi(uint8_t);


/*-------------------------------------------------------------------------------------------------------
 *  void cc1100driver_spiwriteburstreg(uint8_t, unsigned char*, uint8_t)
 *
 *  DESCRIPTION:
 *      This function writes to multiple CC1100 registers, using SPI burst access.
 *
 *  ARGUMENTS:
 *      uint8_t 
 *          Address of the first CC1100 register to be accessed.
 *      char*
 *          Array of bytes to be written into a corresponding range of
 *          CC1100 registers, starting by the address specified in _addr_.
 *      uint8_t 
 *          Number of bytes to be written to the subsequent CC1100 registers.
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_spiwriteburstreg(uint8_t, unsigned char*, uint8_t);


/*-------------------------------------------------------------------------------------------------------
 *  void cc1100driver_spireadburstreg(uint8_t, char*, uint8_t)
 *
 *  DESCRIPTION:
 *      This function reads multiple CC1100 registers, using SPI burst access.
 *
 *  ARGUMENTS:
 *      uint8_t
 *          Address of the first CC1100 register to be accessed.
 *      char*
 *          Pointer to a byte array which stores the values read from a
 *          corresponding range of CC1100 registers.
 *      uint8_t
 *          Number of bytes to be read from the subsequent CC1100 registers.
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_spireadburstreg(uint8_t, char*, uint8_t);


/*-------------------------------------------------------------------------------------------------------
 *  void cc1100driver_initinterrupts(void)
 *
 *  DESCRIPTION:
 *      This initialises the Microcontroller to provide the needed interrupts.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_initinterrupts(void);


/*-------------------------------------------------------------------------------------------------------
 *  void cc1100driver_beforesend(void)
 *
 *  DESCRIPTION:
 *      This function is called directly before the paket is transfered into the cc1100 to do some
 *		Initalisations etc.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_beforesend(void);


/*-------------------------------------------------------------------------------------------------------
 *  void drivercc1100_load(callback_t, callback_t)
 *
 *  DESCRIPTION:
 *      This function initalizes the hardware and sets up the driver.
 *
 *  ARGUMENTS:
 *     callback_t 
 *          This callback is invoked if a carrier was sensed.
 *			This is called out of an interrupt. So keep the invoked function smart!
 *     callback_t 
 *          This callback is invoked if a paket was received by the cc1100. No further checks
 *			will be done. This is called out of an interrupt. So keep the invoked function smart!
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_load(callback_t, callback_t);

/*-------------------------------------------------------------------------------------------------------
 *  void drivercc1100_aftersend(void)
 *
 *  DESCRIPTION:
 *      This function is called directly after the paket is transfered into the cc1100 to do some
 *		hardware related switching etc.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_aftersend(void);

/*-------------------------------------------------------------------------------------------------------
 *  void drivercc1100_enablecsdetect(void)
 *
 *  DESCRIPTION:
 *      This function enables the detection of a carrier. If one is detected cs_cb is called.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_enablecsdetect(void);

/*-------------------------------------------------------------------------------------------------------
 *  vvoid drivercc1100_disablecsdetect(void)
 *
 *  DESCRIPTION:
 *      This function disables the detection of a carrier signal.
 *
 *-------------------------------------------------------------------------------------------------------
 */
extern void drivercc1100_disablecsdetect(void);

#endif /*DRIVER_CC1100_H_*/
