/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "driver_cc1100.h"
#include "driver_timer.h"
#include "driver_system.h"
#include "spi0.h"
#include "pa.h"
#include "cc1100_defines.h"
#include "driver_controller.h"
#include "smallib.h"

void __attribute__ ((interrupt("IRQ"))) WOR_ISR(void);
void __attribute__ ((interrupt("IRQ"))) CS_ISR(void);

static callback_t _paket_cb;
static callback_t _cs_cb;

//-------------------------------------------------------------------------------------------------------
// CC1100 communication functions (SPI)
//-------------------------------------------------------------------------------------------------------

void drivercc1100_spiinittrx(void)
{
	// Switch to GDO mode output
	PINSEL0&=~(BIT11+BIT10);
	FIO0DIR|= BIT5;
	// CS to low
	CC1100_CS_LOW;
	// Wait for SO to go low (voltage regulator
	// has stabilized and the crystal is running)
	uint32_t t=*TICKS()+20000;
	while (CC1100_GDO1)
	{
		if (t<*TICKS())
			sl_printf("Hanging in SpiInitTrx\n");
	}
	// Switch to SPI mode
	PINSEL0 |= BIT10;
}

//-------------------------------------------------------------------------------------------------------
//  uint8_t trx_spi(uint8_t data)
//
//  DESCRIPTION:
//		This function sends and receives a byte via SPI.
//
//  ARGUMENTS:
//      uint8_t data
//          Data to be send via SPI.
//
//  RETURN VALUE:
//      uint8
//          Return value from CC1100 (SO Pin).
//-------------------------------------------------------------------------------------------------------
uint8_t drivercc1100_trxspi(uint8_t data)
{
	S0SPDR = data; // Schreibe 'data' ins Datenregister
	while (!(S0SPSR & BIT7)); // Warte, bis der Datentransfer beendet ist 
	uint8_t tmp= (uint8_t) S0SPDR;
	return tmp;
}

//-------------------------------------------------------------------------------------------------------
//  void spiReadBurstReg(uint8_t addr, char *buffer, uint8_t count)
//
//  DESCRIPTION:
//      This function reads multiple CC1100 registers, using SPI burst access.
//
//  ARGUMENTS:
//      uint8_t addr
//          Address of the first CC1100 register to be accessed.
//      char *buffer
//          Pointer to a byte array which stores the values read from a
//          corresponding range of CC1100 registers.
//      uint8_t count
//          Number of bytes to be read from the subsequent CC1100 registers.
//-------------------------------------------------------------------------------------------------------
void drivercc1100_spireadburstreg(uint8_t addr, char *buffer, uint8_t count)
{
	uint8_t i;
	drivercc1100_spiinittrx();
	drivercc1100_trxspi(addr | CC1100_READ_BURST);
	for (i = 0; i < count; i++)
	{
		S0SPDR = NOBYTE; // Schreibe 'data' ins Datenregister
		while (!(S0SPSR & BIT7)); // Warte, bis der Datentransfer beendet ist 
		buffer[i] = (uint8_t) S0SPDR;
	}
	CC1100_CS_HIGH;
}

//-------------------------------------------------------------------------------------------------------
//  void spiWriteBurstReg(uint8_t addr, char *buffer, uint8_t count)
//
//  DESCRIPTION:
//      This function writes to multiple CC1100 registers, using SPI burst access.
//
//  ARGUMENTS:
//      uint8_t addr
//          Address of the first CC1100 register to be accessed.
//      char *buffer
//          Array of bytes to be written into a corresponding range of
//          CC1100 registers, starting by the address specified in _addr_.
//      uint8_t count
//          Number of bytes to be written to the subsequent CC1100 registers.
//-------------------------------------------------------------------------------------------------------
void drivercc1100_spiwriteburstreg(uint8_t addr,unsigned char *buffer, uint8_t count)
{
	uint8_t i;
	drivercc1100_spiinittrx();
	S0SPDR = (unsigned char)(addr + (unsigned char)CC1100_WRITE_BURST); 		// Schreibe 'data' ins Datenregister
	for (i = 0; i < count; i++)
	{
		while (!(S0SPSR & BIT7)); 	// Warte, bis der Datentransfer beendet ist
		S0SPDR = buffer[i]; 		// Schreibe 'data' ins Datenregister
	}
	while (!(S0SPSR & BIT7)); 	// Warte, bis der Datentransfer beendet ist
	CC1100_CS_HIGH;
}

void drivercc1100_load(callback_t cs_cb,callback_t paket_cb)
{
	_paket_cb = paket_cb;
	_cs_cb = cs_cb;
	spi0_init();	
}

void drivercc1100_aftersend(void)
{
	PaOff();
	LnaOn();
	EXTINT=BIT2;
	VICIntEnable=BIT16;
}

void drivercc1100_initinterrupts(void)
{
// Enable interrupts for packet reception on GDO2
	disableIRQ();			// Disable all interrupts
	PINSEL0|=BIT31; 		// Select EINT2 function for GDO2 Pin
	EXTMODE|= BIT0+BIT2;	// Edge Sensitivy
	VICIntEnable = BIT16; 	// Enable Interrupthandling for EINT2
	VICVectCntl0 = 16+ BIT5;// Assign EINT2 to IRQ Slot 0
	VICVectAddr0 = (unsigned long)WOR_ISR;// Assing Isr Address
	//EXTPOLAR|=BIT2;	// EINT2 on rising edge
	INTWAKE|= BIT2;	// Wakes the uC from PowerDown on GDO2 interrupt
	
	// Enable interrupts for CarrierSense on GD0 but don't activate it
	PINSEL1|=BIT0; 		// Select EINT0 function for GDO0 Pin
	VICVectCntl1 = 14+ BIT5;// Assing EINT0 to IRQ Slot 0
	VICVectAddr1 = (unsigned long)CS_ISR;// Assing Isr Address
	EXTPOLAR|=BIT0+BIT2;	// EINT0 on rising edge
	
	EXTINT=BIT0;
	EXTINT=BIT2;
	
	enableIRQ();			// Enable all interrupts
}

void drivercc1100_beforesend(void)
{
	// Disable interrupts on GDO2
	VICIntEnClear=BIT16;
	LnaOff();
	PaOn(PA_POWER_30DBM);
}

void drivercc1100_enablecsdetect(void)
{
	EXTINT=BIT0;								// Clear IFG for GDO0 EINT0
	VICIntEnable=BIT14;							// Enable interrupt for GDO0
}

void drivercc1100_disablecsdetect(void)
{
	EXTINT=BIT0;								// Clear IFG for GDO0 EINT0
	VICIntEnClear=BIT14;							// Disable interrupt for GDO0
}

void __attribute__ ((interrupt("IRQ"))) CS_ISR(void)
{
	_cs_cb();
	VICIntEnClear=BIT14;	// Disable interrupt for GDO0
	EXTINT=BIT0;			// Clear IFG for GDO0
	VICVectAddr=0; 			// Dummy write to signal end of interrupt
}

void __attribute__ ((interrupt("IRQ"))) WOR_ISR(void)
{
	if (system_state.POWERDOWN) 
	{
		init_clks();
		system_state.POWERDOWN = false;
	}
	while (FIO0PIN&BIT15);
	_paket_cb();
	EXTINT=BIT2; //clear IF Flag
	VICVectAddr=0; //Dummy write to signal end of interrupt
}
