/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"
#include "driver_sensor.h"

#ifdef INCLUDE_SHT11

#include <msp430x16x.h>
#include "driver_sht11.h"
#include "type.h"
#include "driver_timer.h"

//----------------------------------------------------------------------------------
// modul-var
//----------------------------------------------------------------------------------
enum {TEMP, HUMI};

/* Setting the offset of the bandgap temperature sensor according to the datasheet */
// #define SHT11_TEMP_OFFSET		(40)  // @ 5V
// #define SHT11_TEMP_OFFSET (39.75)  // @ 4V
// #define SHT11_TEMP_OFFSET (39.66)  // @ 3.5V
#define SHT11_TEMP_OFFSET (39.60)  // @ 3V
// #define SHT11_TEMP_OFFSET (39.55)  // @ 2.5V

#define SHT11_SCK_LOW	(P3OUT &= ~0x20)		// serial clock line low
#define SHT11_SCK_HIGH	(P3OUT |= 0x20)			// serial clock line high

#define SHT11_DATA		((P3IN & 0x10)>>4)		// read serial I/O
#define SHT11_DATA_LOW	(P3OUT &= ~0x10)		// serial I/O line low
#define SHT11_DATA_HIGH	(P3OUT |= 0x10)			// serial I/O line high
#define SHT11_DATA_IN	(P3DIR &= ~0x10)		// serial I/O as input
#define SHT11_DATA_OUT	(P3DIR |= 0x10)			// serial I/O as output

#define SHT11_NO_ACK	(0)
#define SHT11_ACK		(1)

							 		//adr command  r/w
#define SHT11_STATUS_REG_W	(0x06) 	//000  0011    0
#define SHT11_STATUS_REG_R	(0x07) 	//000  0011    1
#define SHT11_MEASURE_TEMP	(0x03) 	//000  0001	   1
#define SHT11_MEASURE_HUMI	(0x05) 	//000  0010	   1
#define SHT11_RESET			(0x1E) 	//000  1111	   0

s_sht11_data sht11_data;

//----------------------------------------------------------------------------------
//  char sht11_write_byte(uint8_t value)
//
//	DESCRIPTION:
//		Writes a byte on the Sensibus and checks the acknowledge.
//
//----------------------------------------------------------------------------------
char sht11_write_byte(uint8_t value) {
	uint8_t i, error = 0;
	SHT11_DATA_OUT;					// SHT11_DATA is output
	_NOP();
	for (i = 0x80; i > 0; i /= 2) {	// Shift bit for masking
		if (i & value)
			SHT11_DATA_HIGH;		// Masking value with i, write to SENSI-BUS
		else
			SHT11_DATA_LOW;
		SHT11_SCK_HIGH;				// CLK for SENSI-BUS
		_NOP(); // pulswith
		_NOP(); // approx.
		_NOP(); // 5 us
		SHT11_SCK_LOW;
		_NOP();
		_NOP();
	}
	SHT11_DATA_HIGH;	// Release DATA-line
	SHT11_DATA_IN;		// SHT11_DATA is input
	_NOP();
	SHT11_SCK_HIGH;		// CLK #9 for ACK
	_NOP();
	_NOP();
	error = SHT11_DATA;	// Check ACK (DATA will be pulled down by SHT11)
	SHT11_SCK_LOW;
	return error;		// Error = 1 in case of no acknowledge
}

//----------------------------------------------------------------------------------
//  char sht11_read_byte(uint8_t ack)
//
//	DESCRIPTION:
//		Reads a byte form the Sensibus and gives an acknowledge in case of "ack=1".
//
//----------------------------------------------------------------------------------
char sht11_read_byte(uint8_t ack) {
	uint8_t i, val = 0;
	SHT11_DATA_OUT;					// SHT11_DATA is output
	_NOP();
	SHT11_DATA_HIGH;				// Release DATA-line
	SHT11_DATA_IN;					// SHT11_DATA is input
	_NOP();
	for (i = 0x80; i > 0; i /= 2) {	// Shift bit for masking
		SHT11_SCK_HIGH;				// CLK for SENSI-BUS
		_NOP();
		_NOP();
		if (SHT11_DATA)
			val = (val | i);		// Read bit
		SHT11_SCK_LOW;
		_NOP();
		_NOP();
	}
	SHT11_DATA_OUT;					// SHT11_DATA is output
	_NOP();
	if (ack) SHT11_DATA_LOW;		// In case of "ack==1" pull down DATA-Line
	else SHT11_DATA_HIGH;			// else pull up.
	SHT11_SCK_HIGH;					// CLK #9 for ACK
	_NOP();							// pulswith
	_NOP();							// approx.
	_NOP();							// 5 us
	SHT11_SCK_LOW;
	_NOP();
	_NOP();
	SHT11_DATA_HIGH;				// Release DATA-line
	return val;
}

//----------------------------------------------------------------------------------
//  void sht11_transstart(void)
//
//	DESCRIPTION:
//		Generates a transmission start.
//
//		 _____         ________
// DATA:	  |_______|
//           ___     ___
// SCK : ___|   |___|   |______
//
//----------------------------------------------------------------------------------
void sht11_transstart(void) {
	SHT11_DATA_OUT;		// SHT11_DATA is output
	_NOP();
	SHT11_DATA_HIGH;	// Initial state
	SHT11_SCK_LOW;		// Initial state
	_NOP();
	_NOP();
	SHT11_SCK_HIGH;
	_NOP();
	_NOP();
	SHT11_DATA_LOW;
	_NOP();
	_NOP();
	SHT11_SCK_LOW;
	_NOP();
	_NOP();
	_NOP();
	_NOP();
	_NOP();
	_NOP();
	SHT11_SCK_HIGH;
	_NOP();
	_NOP();
	SHT11_DATA_HIGH;
	_NOP();
	_NOP();
	SHT11_SCK_LOW;
}

//----------------------------------------------------------------------------------
//  void sht11_connectionreset(void)
//
//	DESCRIPTION:
//		Communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart.
//
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
//
//----------------------------------------------------------------------------------
void sht11_connectionreset(void) {
	uint8_t i;
	SHT11_DATA_OUT;				// SHT11_DATA is output
	_NOP();
	SHT11_DATA_HIGH;			// Initial state
	SHT11_SCK_LOW;				// Initial state
	for(i = 0; i < 9; i++) {	// 9 SCK cycles
		SHT11_SCK_HIGH;
		_NOP();
		_NOP();
		SHT11_SCK_LOW;
		_NOP();
		_NOP();
	}
	sht11_transstart();			// Transmission start
}

//----------------------------------------------------------------------------------
//  char sht11_softreset(void)
//
//	DESCRIPTION:
//		Resets the sensor by a softreset.
//
//----------------------------------------------------------------------------------
char sht11_softreset(void) {
	uint8_t error = 0;
	sht11_connectionreset();				// Reset communication
	error += sht11_write_byte(SHT11_RESET); // Send RESET-command to sensor
	return error;							// Error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
//  char sht11_read_statusreg(uint8_t *p_value, uint8_t *p_checksum)
//
//	DESCRIPTION:
//		Reads the status register with checksum (8-bit).
//
//----------------------------------------------------------------------------------
char sht11_read_statusreg(uint8_t *p_value, uint8_t *p_checksum) {
	uint8_t error = 0;
	sht11_transstart();								// Transmission start
	error = sht11_write_byte(SHT11_STATUS_REG_R);	// Send command to sensor
	*p_value = sht11_read_byte(SHT11_ACK);			// Read status register (8-bit)
	*p_checksum = sht11_read_byte(SHT11_NO_ACK);	// Read checksum (8-bit)
	return error;									// Error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
//  char sht11_write_statusreg(uint8_t *p_value)
//
//	DESCRIPTION:
//		Writes the status register with checksum (8-bit).
//
//----------------------------------------------------------------------------------
char sht11_write_statusreg(uint8_t *p_value) {
	uint8_t error = 0;
	sht11_transstart();								// Transmission start
	error += sht11_write_byte(SHT11_STATUS_REG_W);	// Send command to sensor
	error += sht11_write_byte(*p_value);			// Send value of status register
	return error;									// Error>=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
//  char sht11_measure(sht_value *p_sht_value, uint8_t *p_checksum, uint8_t mode)
//
//	DESCRIPTION:
//		Makes a measurement (humidity/temperature) with checksum.
//
//----------------------------------------------------------------------------------
char sht11_measure(sht_value *p_sht_value, uint8_t *p_checksum, uint8_t mode) {
	uint8_t i, error = 0;
	sht_value sht_value_temp;
	sht11_transstart(); // Transmission start
	switch (mode) {		// Send command to sensor
		case TEMP: error += sht11_write_byte(SHT11_MEASURE_TEMP); break;
		case HUMI: error += sht11_write_byte(SHT11_MEASURE_HUMI); break;
		default: break;
	}
	for (i = 0; i < 3; i++) {
		_wait(1750);
		if (SHT11_DATA == 0) break; // Wait until sensor has finished the measurement (~210 ms for 14 bit)
	}
	if (SHT11_DATA) error += 1;								// Or timeout is reached
	sht_value_temp.i =  256 * sht11_read_byte(SHT11_ACK);	// Read the first byte (MSB)
	sht_value_temp.i += sht11_read_byte(SHT11_ACK);			// Read the second byte (LSB)
	*p_checksum = sht11_read_byte(SHT11_NO_ACK);			// Read checksum
	*(p_sht_value) = sht_value_temp;
	return error;
}

//----------------------------------------------------------------------------------
//  void sht11_calc_th(sht_value *p_humidity, sht_value *p_temperature)
//
//	DESCRIPTION:
//		Calculates temperature [�C] and humidity [%RH].
//
//		input : humi [Ticks] (12 bit)
//				temp [Ticks] (14 bit)
//		output: humi [%RH]
//				temp [�C]
//
//----------------------------------------------------------------------------------
void sht11_calc_th(sht_value *p_humidity, sht_value *p_temperature) {
	const float C1 = -4.0;			// for 12 Bit
	const float C2 = 0.0405;		// for 12 Bit
	const float C3 = -0.0000028;	// for 12 Bit
	const float T1 = 0.01;			// for 14 Bit @ 5V
	const float T2 = 0.00008;		// for 14 Bit @ 5V

	float rh_lin;				// rh_lin:	Humidity linear
	float rh_true;				// rh_true: Temperature compensated humidity
	float t_C;					// t_C :	Temperature [�C]

	t_C = 0.01 * p_temperature->i - SHT11_TEMP_OFFSET;						// Calc. Temperature from ticks to [�C]
	rh_lin = C3 * p_humidity->i * p_humidity->i + C2 * p_humidity->i + C1;	// Calc. Humidity from ticks to [%RH]
	rh_true = (t_C - 25) * (T1 + T2 * p_humidity->i) + rh_lin;				// Calc. Temperature compensated humidity [%RH]
	if (rh_true > 100) rh_true = 100;										// Cut if the value is outside of
	if (rh_true < 0.1) rh_true = 0.1;										// the physical possible range
	p_temperature->f = t_C;													// Return temperature [�C]
	p_humidity->f = rh_true;												// Return humidity [%RH]
}

void sht11_init(driver* this) {
	sht11_connectionreset();
	sht11_data.v_timestamp = 0;
}

void sht11_update_data(void) {
	uint8_t error = 0, checksum;
	uint32_t timespan = *TICKS() - sht11_data.v_timestamp;
	if (timespan < 16384 && sht11_data.v_timestamp!=0) return; // must be >= ~2 sec
	error = 0;
	error += sht11_measure(&sht11_data.humi_val, &checksum, HUMI); // Measure humidity
	error += sht11_measure(&sht11_data.temp_val, &checksum, TEMP); // Measure temperature
	sht11_data.v_timestamp = *TICKS();
	if (error != 0)
		sht11_connectionreset();									// In case of an error: connection reset
	else
		sht11_calc_th(&sht11_data.humi_val, &sht11_data.temp_val);	// Calculate humidity, temperature
}

#endif

float sht11_humi_get(sensor* this){
	sht11_update_data();
	return sht11_data.humi_val.f;
}

float sht11_temp_get(sensor* this){
	sht11_update_data();
	return sht11_data.temp_val.f;
}

sensor* sht11_humi_new(sensor* this){
	this->base.init = sht11_init;
	this->get_value = sht11_humi_get;
	return this;
}


sensor* sht11_temp_new(sensor* this){
	this->base.init = sht11_init;
	this->get_value = sht11_temp_get;
	return this;
}


