/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef NET_H
#define NET_H

#include "routing.h"

#define HANDLER_TABLE_SIZE	 (10)

// ---------------------------------------------------------------------------------
// The callback for incomming pakets
// First parameter is the id of the paket sender
// Second Parameter is the raw layer2 (in our model, in OSI-Model this would be 
//		application layer) paket.
// ---------------------------------------------------------------------------------
typedef void (*packet_handler_t)(uint8_t,uint8_t*);



// --------------------------------------------------------
// Public Functions 
// --------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
//  void initNet(void)
//
//  DESCRIPTION:
//		Initialize the net layer. Must be called before the net layer can be used.
//
//-------------------------------------------------------------------------------------------------------
void initNet(void);

//-------------------------------------------------------------------------------------------------------
//	bool net_send(uint8_t type, uint8_t dest, uint8* data)
//
//	DESCRIPTION:
//		Queues the data for transmission in the txBuffer of the routing layer.
//		Data is transmitted as soon as a route to the given destination is known.
//
//  ARGUMENTS:
//		uint8_t type
//			The application specific protocol id for the dsr layer (must be >= 127,
//			other ids reserved for routing).
//		uint8_t dest
//			Destination address (must not be equal to this nodes uid).
//		UINT8* data
//			Pointer to the data to be transmitted.
//
//  RETURN VALUE:
//		bool
//			true:	If data could be stored in TX buffer for transmission.
//			false:	If packet couldn't be stored in TX buffer (buffer full) or
//					_dest_ was equal to the nodes uid.
//
//-------------------------------------------------------------------------------------------------------
bool net_send(uint8_t type, uint8_t dest, uint8_t* data);

//-------------------------------------------------------------------------------------------------------
//  bool registerPacketHandler(uint8_t type, packet_handler_t handler)
//
//  DESCRIPTION:
//		Register a packet handler of the given protocol type. If already
//		a handler exists, the new one will be taken.
//
//  ARGUMENTS:
//		uint8_t type
//			The protocol identifier. Must be >= 127.
//		packet_handler_t handler
//			The packet handler function which is called if a packet with
//			the given type is received.
//
//  RETURN VALUE:
//		bool
//			true:	If packet handler registration was successful.
//			false:  If packet handler registration failed because the handler table is full
//					or the given type is invalid (<=127).
//
//-------------------------------------------------------------------------------------------------------
bool registerPacketHandler(uint8_t type, packet_handler_t handler);

//-------------------------------------------------------------------------------------------------------
//  bool unregisterPacketHandler(uint8_t type)
//
//  DESCRIPTION:
//		Removes the packet handler for the given protocol type.
//
//  ARGUMENTS:
//		uint8_t type
//			The protocol type.
//
//  RETURN VALUE:
//		bool
//			true:	If packet handler was successfully removed.
//			false:  If no packet handler for that type existed.
//
//-------------------------------------------------------------------------------------------------------
bool unregisterPacketHandler(uint8_t type);



// --------------------------------------------------------
// Net-Api internal Functions (normaly used by routing.c)
// --------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
//  void net_packet_handler(packet_layer1_t* packet)
//
//  DESCRIPTION:
//		Packet handler of the net layer. Handles all packets not handled by the routing layer.
//		This is only called by the routing Layer. Not by the user!
//
//  ARGUMENTS:
//		packet_layer1_t* packet
//			Pointer to received packet (of routing layer).
//
//-------------------------------------------------------------------------------------------------------
void net_packet_handler(packet_layer1_t* packet);

#endif
