/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef CC1100_ROUTING_H
#define CC1100_ROUTING_H

#include "funk.h"

#define ADDR_BROADCAST	(0x00)		// Broadcast address
#define READ_CFLAG(x)	(x & 0x80) 	// Makro for reading the C-Flag out of the flags field
#define WRITE_CFLAG(x)	(x | 0x80) 	// Makro for writing the C-Flag in the flags field

#define P_DATA  (127)	// Base index for application protocols (data for routing layer)

#define COMMAND_RESET			(1)	// Reset command
#define COMMAND_OUTPUT_POWER	(2)	// Set output power command

//-----------------------------------------------------------------------------------------
// Layer 1 protocol (routing layer "AAODV"):
//
// ----------------------------------------------------------------------------------------
// |          |             |        |     |                |       |            |        |
// | Protocol | Destination | Source | TTL | Identification | Flags | SeqClearId |  Data  |
// |          |             |        |     |                |       |            |        |
// ----------------------------------------------------------------------------------------
//    1 byte      1 byte      1 byte  1 byte     2 bytes      1 byte   1 byte     <= 50 bytes
//
// Flags:
//			Bit | Meaning
//			--------------------
//			7	| C-Flag (Clear sequence number flag)
//			6:0 | Reserved
//
//-----------------------------------------------------------------------------------------
typedef struct  __attribute__ ((packed)) p_layer1
{
	uint8_t protocol;
	uint8_t destination;
	uint8_t source;
	uint8_t ttl;
	uint16_t identification;
	uint8_t flags;
	uint8_t seqClearId;
	uint8_t data[50];
} packet_layer1_t;

typedef void (*app_callback_t)(packet_layer1_t*);

//-------------------------------------------------------------------------------------------------------
//  void initRouting(app_callback_t callback)
//
//  DESCRIPTION:
//		Initialize the routing layer. Must be called before the routing can be used.
//
//  ARGUMENTS:
//		app_callback_t callback
//			A callback function of the application layer which handles incoming packets.
//
//-------------------------------------------------------------------------------------------------------
void initRouting(app_callback_t callback);

//-------------------------------------------------------------------------------------------------------
//	bool sendLayer1Data(UINT8* data, uint8_t appProtocol, uint8_t dest)
//
//	DESCRIPTION:
//		Queues the data for transmission in the txBuffer of the routing layer.
//		Data is transmitted as soon as a route to the given destination is known.
//
//  ARGUMENTS:
//		UINT8* data
//			Pointer to the data to be transmitted.
//		uint8_t appProtocol
//			The application specific protocol id for the AAODV layer (must be >= 127,
//			other ids reserved for routing).
//		uint8_t dest
//			Destination address (must not be equal to this nodes uid).
//
//  RETURN VALUE:
//		bool
//			true:	If data could be stored in TX buffer for transmission.
//			false:	If packet couldn't be stored in TX buffer (buffer full) or
//					_dest_ was equal to the nodes uid.
//
//-------------------------------------------------------------------------------------------------------
bool sendLayer1Data(uint8_t* data, uint8_t appProtocol, uint8_t dest);

//-------------------------------------------------------------------------------------------------------
//	bool sendLayer1Command(uint8_t command, UINT8* data)
//
//	DESCRIPTION:
//		Transmits a command into the network which is executed by all nodes.
//		The packet is not stored in the TX buffer of the node but is send immediately.
//
//  ARGUMENTS:
//		uint8_t command
//			The network command to be executed on the nodes.
//		UINT8* data
//			Additional network command data.
//			
//  RETURN VALUE:
//		bool
//			true:	If packet transmission was successful.
//			false:	If packet couldn't be send.
//
//-------------------------------------------------------------------------------------------------------
bool sendLayer1Command(uint8_t command, uint8_t* data);

//-------------------------------------------------------------------------------------------------------
//  bool rxBufferAdd(packet_layer0_t* packet)
//
//  DESCRIPTION:
//		Adds a packet to the RX buffer (as a copy). The packet is handled by calling the
//		function handlePackets() which should happen in the main loop.
//
//  ARGUMENTS:
//		packet_layer0_t* packet
//			Pointer to the packet which should be copied.
//
//  RETURN VALUE:
//		bool
//			true:	If packet could be stored in RX buffer.
//			false:	If packet couldn't be stored in RX buffer (buffer full).
//-------------------------------------------------------------------------------------------------------
bool rxBufferAdd(packet_layer0_t* packet);

//-------------------------------------------------------------------------------------------------------
//	bool isPacketHandlingNeeded(void)
//
//	DESCRIPTION:
//		Checks if packet handling is needed. If so, handlePackets() must be called.
//		If the result was _true_ after returning from handlePackets() a new check of
//		isPacketHandlingNeeded() is neccessary (see handlePackets()).
//
//  RETURN VALUE:
//		bool
//			true:	If packet handling is needed.
//			false:	If there are no packets to be handled.
//
//-------------------------------------------------------------------------------------------------------
bool isPacketHandlingNeeded(void);

//-------------------------------------------------------------------------------------------------------
//  void handlePackets(void)
//
//  DESCRIPTION:
//		Handles incoming and outgoing packets. This function must be called
//		regularily out of the main loop (with high priority).
//
//		First incoming packets are handled and then outgoing packets (packets of this node).
//		The packet handling functions only handle one packet and then return (rxHandler: first
//		packet in FIFO, txHandler first packet with known route). That's why this function might be
//		called several times in a loop and the main firmware loop can decide which behaviour is wanted
//		(handle all packets or do some other stuff in between).
//-------------------------------------------------------------------------------------------------------
void handlePackets(void);

//-------------------------------------------------------------------------------------------------------
//  void getNextRouteRequestTimeout(uint32* timeout)
//
//  DESCRIPTION:
//		Returns the closest timeout value (next route request) for a pending packet in the tx buffer. If
//		no packet is pending, then the result is 0.
//
//  ARGUMENTS:
//		uint32* timeout
//			The place to store the closest timeout value.
//
//-------------------------------------------------------------------------------------------------------
void getNextRouteRequestTimeout(uint32_t* timeout);

/** Debug stuff */
void printRoutingTable(void);
void printSeqNumBuffer(void);

#endif
