/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef TIMER_H_
#define TIMER_H_
#include "type.h"

// --------------------------------------------------------
// Defines
// --------------------------------------------------------

#define FIRED_TIMERS_SIZE (5)		// The number of timer which can fire at the same time
typedef void (*fp_timer) (void); 	// The callback for a fired timer

// --------------------------------------------------------
// Global Vars
// --------------------------------------------------------

extern volatile fp_timer fired_timers[FIRED_TIMERS_SIZE]; // The array to store fired timers

// --------------------------------------------------------
// Public Functions
// --------------------------------------------------------

//-------------------------------------------------------------------------------------------------------
//  void init_timer0(void)
//
//  DESCRIPTION:
//		Initialises all timer data-structures. Calls driver_timer.load_driver()
//		to init the Hardware.
//
//---------------------------------------------------------------------------------------------------
void init_timer0(void);

//-------------------------------------------------------------------------------------------------------
//  void wait_ticks(unsigned int num)
//
//  DESCRIPTION:
//		A blocking wait. The length of a tick varies on different hardware platforms.
//		The Length of a tick is defined in cc1100_defines.h of the Hardware branch. Use the
//		TIMER_TICK_USEC_RES Define to normalize.
//
//  ARGUMENTS:
//      unsigned int num
//          The total number of ticks to wait.
//
//---------------------------------------------------------------------------------------------------
void wait_ticks(unsigned int num);


//-------------------------------------------------------------------------------------------------------
//  int register_timer(fp_timer fp, unsigned int ticks)
//
//  DESCRIPTION:
//		This functions registers a nonblocking timer. The callback is transfered in the 
//		fired_timers array and *not* called directly. So it could be invoked from outside 
//		an interrupt. The Length of a tick is defined in cc1100_defines.h of the Hardware branch. Use the
//		TIMER_TICK_USEC_RES Define to normalize.
//
//  ARGUMENTS:
//      fp_timer fp
//          A function pointer to be called if the timer procs.
//		unsigned int ticks
//			The number of ticks until the timer procs.
//  RETURN VALUE:
//      int
//          The id of the registered timer. returns 0 on error.
//
//---------------------------------------------------------------------------------------------------
int register_timer(fp_timer fp, unsigned int ticks);

//-------------------------------------------------------------------------------------------------------
//  void delete_timer(int id)
//
//  DESCRIPTION:
//		Deletes and deactivates an active timer
//
//---------------------------------------------------------------------------------------------------
void delete_timer(int id);

//-------------------------------------------------------------------------------------------------------
//  inline uint32_t ticks(void)
//
//  DESCRIPTION:
//		The Number of ticks since system reboot. 
//		TODO: This function is not overflow safe!
//		The Length of a tick is defined in cc1100_defines.h of the Hardware branch. Use the
//		TIMER_TICK_USEC_RES Define to normalize.
//
//  RETURN VALUE:
//      int
//          The Number of ticks since system reboot.
//---------------------------------------------------------------------------------------------------
inline uint32_t ticks(void);

#endif /*TIMER_H_*/
