package carrera;

import carrera.data.CarreraGame;
import carrera.data.CarreraGameEvent;
import carrera.data.CarreraGameListener;
import carrera.data.CarreraGames;
import carrera.data.CarreraPlayer;
import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.text.DecimalFormat;
import java.util.Iterator;
import javax.swing.JPanel;

/**
 *
 * @author hillebra
 */
public class GameCanvas extends JPanel implements CarreraGameListener {

    private Main main;
    
     /** Offscreen image for drawing */
    private Image image;
    
    /** Graphics object of offscreen image */
    private Graphics2D buffer;
    
    /** Aktuelle Y-POS für das Zeichnen */
    private int currentYPos;

    private static final int ALIGN_LEFT = 1;
    private static final int ALIGN_CENTER = 2;
    private static final int ALIGN_RIGHT = 3;
    private static final int ALIGN_BOTTOM = 4;
    private static final int ALIGN_TOP = 5;
    
    private static final Color PAINT_COLOR = Color.GREEN;
    
    public GameCanvas(Main main) {
        this.main = main;
        main.addGameListener(this);
    }
    
    public void carreraGameChanged(CarreraGameEvent event) {
        repaint();
    }
    
    @Override
    protected void paintComponent(Graphics g) {
        if (image == null || (! (image.getWidth(this) == getSize().width
                && image.getHeight(this) == getSize().height))) {
            image = createImage(getSize().width, getSize().height);
            buffer = (Graphics2D) image.getGraphics();
            buffer.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                             RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        }
        
        super.paintComponent(buffer);
        Paint oldPaint = buffer.getPaint();
        
        switch (main.getGameMode()) {
            case CarreraGames.MODE_DISPLAY_BOOTSCREEN:
                paintBootScreen();
                break;
            case CarreraGames.MODE_DISPLAY_CURRENT_GAME:
                paintCurrentGame();
                break;
            case CarreraGames.MODE_DISPLAY_HIGH_SCORE:
                paintHighScore();
                break;
        }
        g.drawImage(image, 0, 0, this);
        buffer.setPaint(oldPaint);
    }
    
    private void paintBootScreen() {
        Image img = new javax.swing.ImageIcon(
                getClass().getResource("/carrera/img/carrera.jpg")).getImage();
        buffer.drawImage(img, 0, 0, getSize().width, getSize().height, null);
    }
    
    private void paintHighScore() {
        
        Composite oldComposite = buffer.getComposite();
        Font oldFont = buffer.getFont();
        
        Image img = new javax.swing.ImageIcon(
                getClass().getResource("/carrera/img/highscore.jpg")).getImage();
        
        buffer.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
        buffer.drawImage(img, 0, 0, getSize().width, getSize().height, null);
        buffer.setComposite(oldComposite);
        
        String s;
        currentYPos = 0;
        int w = getSize().width;
        int h = getSize().height;
        buffer.setPaint(Color.BLACK);
        buffer.setFont(new Font("Courier", Font.BOLD, 60));
        
        s = "Highscore";
        currentYPos += buffer.getFontMetrics().getHeight() + 20;
        buffer.drawString(s, getXPosition(s, buffer.getFontMetrics(), w,
                ALIGN_CENTER), currentYPos);
        currentYPos += buffer.getFontMetrics().getHeight() / 2;
        buffer.setFont(new Font("Courier", Font.BOLD, 42));
        
        int count = 0;
        Iterator<CarreraPlayer> iter = main.getCarreraGames().getHighScore();
        while (iter.hasNext() && count < 10) {
            CarreraPlayer player = iter.next();
            if (player.isAuto()) {
                buffer.setPaint(Color.RED);
            } else {
                buffer.setPaint(Color.BLACK);
            }
            s = padString((count+1) + ".", '0', 3, 1);
            s += " ";
            s += padString(player.getName(), ' ', 15, 0);
            s += " ";
            s += prettyFormatTime(player.getOverallTime());
            currentYPos += buffer.getFontMetrics().getHeight();
            buffer.drawString(s, getXPosition(s, buffer.getFontMetrics(), w,
                ALIGN_CENTER), currentYPos);
            count++;
        }
        
        buffer.setFont(oldFont);
    }

    private void paintCurrentGame() {
        
        currentYPos = 0;
        int w = getSize().width;
        int h = getSize().height;
        
        buffer.setPaint(Color.BLACK);
        buffer.fillRect(0, 0, w, h);
        buffer.setPaint(PAINT_COLOR);
        buffer.setFont(buffer.getFont().deriveFont(20f));
        
        // Aktuelles Spiel laden, falls nicht vorhanden, dann Ende mit Meldung
        CarreraGame game = main.getCurrentGame();
        if (game == null) {
            currentYPos = getYPosition(h, buffer.getFontMetrics().getHeight(),
                    ALIGN_CENTER);
            buffer.drawString("Fehler: Kein Spiel vorhanden!",
                    getXPosition("Fehler: Kein Spiel vorhanden!",
                buffer.getFontMetrics(), w, ALIGN_CENTER), currentYPos);
            return;
        }

        if (game.isInitial()) {
            paintCurrentGame_Initial(game, w, h);
        } else if (game.isStarting()) {
            paintCurrentGame_Starting(game, w, h);
        } else if (game.isRunning()) {
            paintCurrentGame_Running(game, w);
        } else if (game.isOver()) {
            paintCurrentGame_Over(game, w, h);
        }
    }
    
    private void paintCurrentGame_Initial(CarreraGame game, int w, int h) {
        
        String s;
        FontMetrics fm;
        int fontHeight;
        float fontSizeVs = 90f;
        float fontSizeName = 150f;

        buffer.setFont(buffer.getFont().deriveFont(fontSizeVs));
        int fsvs = buffer.getFontMetrics().getHeight();

        buffer.setFont(buffer.getFont().deriveFont(fontSizeName));
        fm = buffer.getFontMetrics();
        fontHeight = fm.getHeight();
        s = game.getAutomaticPlayer().getName();
        currentYPos = getYPosition(h, fontHeight, ALIGN_CENTER) - fontHeight + fsvs/4;
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                currentYPos);

        buffer.setFont(buffer.getFont().deriveFont(fontSizeVs));
        fm = buffer.getFontMetrics();
        fontHeight = fm.getHeight();
        s = "vs.";
        currentYPos += fontHeight + fm.getDescent();
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                currentYPos);

        buffer.setFont(buffer.getFont().deriveFont(fontSizeName));
        fm = buffer.getFontMetrics();
        fontHeight = fm.getHeight();
        s = game.getManualPlayer().getName();
        currentYPos += fontHeight;
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                currentYPos);
    }
    
    private void paintCurrentGame_Starting(CarreraGame game, int w, int h) {
        
        String s;
        FontMetrics fm;
        
        buffer.setFont(buffer.getFont().deriveFont(300f));
        fm = buffer.getFontMetrics();

        s = game.getStartCount() > 0 ? "" + game.getStartCount() : "GO!";
        currentYPos = getYPosition(h, fm.getHeight(), ALIGN_CENTER);
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                currentYPos);
    }
    
    private void paintCurrentGame_Running(CarreraGame game, int w) {
        
        String s;
        buffer.setFont(buffer.getFont().deriveFont(40f));
        FontMetrics fm = buffer.getFontMetrics();
        int fontHeight = fm.getHeight();
        
        // Aktuelle Runde zeichnen
        int currentLap = Math.min(game.getCurrentLap(0), game.getCurrentLap(1));
        s = "Verbleibende Runden: " + currentLap + "/" + game.getLapCount();
        currentYPos += fontHeight;
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER), currentYPos);

        // Renndauer (gesamt) zeichnen
        currentYPos += fontHeight;
        s = prettyFormatTime(game.getRaceDuration());
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER), currentYPos);

        // Gas/Bremsverhalten des Autos zeichnen
        if (main.getVehicleStatus() != -1 && main.isShowBT()) {
            currentYPos += 20;
            Paint old = buffer.getPaint();
            s = main.getVehicleStatus() == 0 ? "BREMSE" : "VOLLGAS";
            int dx = getXPosition(s, fm, w, ALIGN_CENTER);
            buffer.setPaint(main.getVehicleStatus() == 0 ? Color.RED : Color.GREEN);
            buffer.fill3DRect(dx, currentYPos, main.getVehicleStatus() == 0 ? 140 : 210, 40, true);
            buffer.setPaint(Color.WHITE);
            buffer.drawString(s, dx, currentYPos + 35);
            buffer.setPaint(old);
        }
        
        paintLapTimes(game, 0, currentYPos, 40, w);    // Auto
        paintLapTimes(game, 1, currentYPos, w/2 + 40, w);  // Mensch
    }
    
    private void paintLapTimes(CarreraGame game, int driverId,
            int yStart, int xOffset, int w) {
        
        String s;
        buffer.setFont(buffer.getFont().deriveFont(40f));
        FontMetrics fm = buffer.getFontMetrics();
        int fontHeight = fm.getHeight();
        
        // Den Spielernamen zeichnen
        yStart += fontHeight * 2;
        s = game.getPlayer(driverId).getName();
        buffer.drawString(s, getXPosition(s, fm, w, ALIGN_LEFT) + xOffset,
                yStart);
        
        buffer.setFont(buffer.getFont().deriveFont(30f));
        fm = buffer.getFontMetrics();
        fontHeight = fm.getHeight();
        
        // Aktuelle Rundenzeit der laufenden Runde für die Fahrer zeichnen
        if (game.isRunning()) {
            buffer.setPaint(Color.WHITE);
            yStart += fontHeight * 2;
            s = "Runde #" + (CarreraGame.MAX_LAPS - game.getCurrentLap(driverId) + 1) + ": "
                    + prettyFormatTime2(game.getCurrentLapTime(driverId)) + " s";
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_LEFT) + xOffset,
                    yStart);
            buffer.setPaint(PAINT_COLOR);
        }
        
        // Die bisherigen Rundenzeiten zeichnen
        if (game.getLapTimes(driverId).size() > 0) {
            int idx = 1;
            synchronized (game.getLapTimes(driverId)) {
                Iterator<Long> iter = game.getLapTimes(driverId).iterator();
                while (iter.hasNext()) {
                    yStart += fontHeight;
                    s = "Runde #" + idx + ": " + prettyFormatTime2(iter.next()) + " s";
                    buffer.drawString(s, getXPosition(s, fm, w, ALIGN_LEFT)
                            + xOffset, yStart);
                    idx++;
                }
            }
        }
    }
    
    private void paintCurrentGame_Over(CarreraGame game, int w, int h) {
        
        String s;
        buffer.setFont(buffer.getFont().deriveFont(100f));
        FontMetrics fm = buffer.getFontMetrics();
        int fontHeight = fm.getHeight();
        
        currentYPos = getYPosition(h, fontHeight, ALIGN_CENTER);
        
        if (game.wasJumpTheGun()) {
            s = "Fruehstart";
            currentYPos -= fontHeight;
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                    currentYPos);
            currentYPos += fontHeight;
            currentYPos += fontHeight;
        }
        
        if (!game.wasSuccess()) {
            s = "GAME OVER";
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                    currentYPos);
        }
        
        if (game.wasSuccess()) {
            s = "Sieger: " + game.getWinner().getName();
            currentYPos -= 1.5*fontHeight;
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                    currentYPos);
            
            // Aktuelle Zeit des Siegers anzeigen
            buffer.setFont(buffer.getFont().deriveFont(50f));
            fm = buffer.getFontMetrics();
            fontHeight = fm.getHeight();
            
            currentYPos += 3*fontHeight;
            s = "Zeit: " + prettyFormatTime(game.getWinner().getOverallTime());
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                    currentYPos);
            currentYPos -= 3*fontHeight;
            
            buffer.setFont(buffer.getFont().deriveFont(100f));
            fm = buffer.getFontMetrics();
            fontHeight = fm.getHeight();
            
            // Aktuellen Rang anzeigen
            int rank = main.getCarreraGames().getHighScorePosition(game.getWinner());
            s = "Highscore: ";
            s += rank != -1 ? ("#" + rank) : "-";
            currentYPos += 3*fontHeight;
            buffer.drawString(s, getXPosition(s, fm, w, ALIGN_CENTER),
                    currentYPos);
        }
    }
    
    private int getXPosition(String s, FontMetrics fm, int totalWidth, int alignment) {
        switch (alignment) {
            case ALIGN_RIGHT:
                return totalWidth - fm.stringWidth(s) - 20;
            case ALIGN_CENTER:
                return (totalWidth - fm.stringWidth(s)) / 2;
            default:
                return 20;
        }
    }
    
    private int getYPosition(int availableHeight, int totalHeight, int alignment) {
        switch (alignment) {
            case ALIGN_BOTTOM:
                return availableHeight - totalHeight - 20;
            case ALIGN_CENTER:
                return ((availableHeight + totalHeight) / 2 - totalHeight / 4);
            default:
                return 20;
        }
    }
    
    /**
     * Formatiert eine Zeitdauerangabe.
     *
     * @param millis Zeitdauer in Millisekunden.
     *
     * @return Formatierte Zeit in [T] H:M:S.MS.
     */
    private static String prettyFormatTime(long millis) {
        long sec = millis / 1000;
        long milli = millis - sec * 1000;
        long min = sec / 60;
        sec -=  min * 60;
        long hour = min / 60;
        min -=  hour * 60;
        long day = hour / 24;
        hour -= day * 24;
        DecimalFormat df = new DecimalFormat("00");
        String s = df.format(hour) + ":" + df.format(min) + ":" + df.format(sec);
        df = new DecimalFormat("000");
        s += "." + df.format(milli);
        return s;
    }
    
    private static String prettyFormatTime2(long millis) {
        long sec = millis / 1000;
        long milli = millis - sec * 1000;
        long min = sec / 60;
        sec -=  min * 60;
        long hour = min / 60;
        min -=  hour * 60;
        /*
        long day = hour / 24;
        hour -= day * 24; */
        DecimalFormat df = new DecimalFormat("00");
        String s = df.format(min) + ":" + df.format(sec);
        df = new DecimalFormat("000");
        s += "." + df.format(milli);
        return s;
    }
    
    /**
     * Stellt Füllzeichen voran, falls die Länge der Zeichenkette kleiner als die
     * übergebene Länge ist.
     * <p>
     * Die Anzahl der Füllzeichen ergibt sich aus der Differenz von
     * <code>len</code> und der Anzahl der Stellen der Zeichenkette.
     *
     * @param s Die Zeichenkette, welche aufgefüllt werden soll.
     * @param pad Füllzeichen.
     * @param len Länge nach Auffüllung - falls nötig.
     * @param padStyle <code>1</code> falls von Links aufgefüllt werden soll;
     *        <code>0</code> falls von Rechts aufgefüllt werden soll.
     *
     * @return Aufgefüllte Zeichenkette.
     */
    private static String padString(String s, char pad, int len, int padStyle) {
        int currentLength = s.length();
        while (currentLength < len) {
            if (padStyle == 1) {
                s = pad + s;
            } else {
                s = s + pad;
            }
            currentLength++;
        }
        
        return s;
    }

}
