package carrera.data;

import carrera.Main;
import java.util.List;

/**
 *
 * @author hillebra
 */
public class CarreraGame implements Comparable<CarreraGame>, Runnable, java.io.Serializable {
    
    private transient Thread t;
    private transient Main main;
    
    private volatile int status;
    private volatile int startCount;
    
    private boolean success;
    private boolean jumpTheGun;
    private CarreraPlayer auto;
    private CarreraPlayer manual;
    private CarreraPlayer winner = null;
    private long raceStartTimestamp = -1;
    private long raceEndTimestamp = -1;
    
    private static int INITIAL = 1;
    private static int STARTING = 2;
    private static int STARTED = 3;
    private static int STOPPED = 4;
    
    private static int COUNT_DOWN = 3;
    public static int MAX_LAPS = 5;
    public static final int DRIVER_CAR = 0;
    public static final int DRIVER_HUMAN = 1;
    public static final String CAR = "CAR";
    
    public CarreraGame(Main main, String autoName, String manualName) {
        this.main = main;
        status = INITIAL;
        success = false;
        auto = new CarreraPlayer(autoName, MAX_LAPS, true);
        manual = new CarreraPlayer(manualName, MAX_LAPS, false);
    }
    
    public CarreraPlayer getAutomaticPlayer() {
        return auto;
    }
    
    public CarreraPlayer getManualPlayer() {
        return manual;
    }
    
    public CarreraPlayer getWinner() {
        return winner;
    }
    
    public int getLapCount() {
        return MAX_LAPS;
    }
    
    public CarreraPlayer getPlayer(int driverId) {
        switch (driverId) {
            case DRIVER_CAR:
                return getAutomaticPlayer();
            case DRIVER_HUMAN:
                return getManualPlayer();
            default:
                return null;
        }
    }
    
    public int getCurrentLap(int driverId) {
        switch (driverId) {
            case DRIVER_CAR:
                return auto.getCurrentLap();
            case DRIVER_HUMAN:
                return manual.getCurrentLap();
            default:
                return -1;
        }
    }
    
    public List<Long> getLapTimes(int driverId) {
        switch (driverId) {
            case DRIVER_CAR:
                return auto.getLapTimes();
            case DRIVER_HUMAN:
                return manual.getLapTimes();
            default:
                return null;
        }
    }
    
    public long getCurrentLapTime(int driverId) {
        if (status != STARTED) return 0;
        long tmp = System.currentTimeMillis();
        switch (driverId) {
            case DRIVER_CAR:
                return tmp - auto.getCurrentLapStartTime();
            case DRIVER_HUMAN:
                return tmp - manual.getCurrentLapStartTime();
            default:
                return 0;
        }
    }
    
    public void decreaseLapCount(int driverId) {
        long tmp = System.currentTimeMillis();
        if (isOver()) return;
        jumpTheGun = !isRunning();
        switch (driverId) {
            case DRIVER_CAR:
                decreaseLapCount0(auto, tmp);
                break;
            case DRIVER_HUMAN:
                decreaseLapCount0(manual, tmp);
                break;
        }
    }
    
    private void decreaseLapCount0(CarreraPlayer p, long ts) {
        if (jumpTheGun) {
            main.endGame(false);
            return;
        }
        if (!p.isFirstLapDecreased()) {
            p.setFirstLapDecreased(true);
            return;
        }
        p.addLapTime(ts - p.getCurrentLapStartTime());
        p.setCurrentLapStartTime(ts);
        p.decreaseCurrentLap();
        if (p.getCurrentLap() == 0) {
            winner = p;
            main.endGame(true);
        }
    }
    
    public void start() {
        if (status != INITIAL) return;
        startCount = COUNT_DOWN;
        t = new Thread(this);
        t.start();
    }
    
    public void end(boolean success) {
        if (status == STOPPED) return;
        raceEndTimestamp = System.currentTimeMillis();
        status = STOPPED;
        this.success = success;
        if (t != null) t.interrupt();
    }

    public boolean isInitial() {
        return (status == INITIAL);
    }
    
    public boolean isStarting() {
        return (status == STARTING);
    }
    
    public boolean isRunning() {
        return (status == STARTED);
    }
    
    public boolean isOver() {
        return (status == STOPPED);
    }
    
    public boolean wasSuccess() {
        return success;
    }
    
    public boolean wasJumpTheGun() {
        return jumpTheGun;
    }
    
    public long getRaceDuration() {
        if (isOver()) return (raceEndTimestamp - raceStartTimestamp);
        if (isRunning()) return (System.currentTimeMillis() - raceStartTimestamp);
        return 0; 
    }

    public int getStartCount() {
        return startCount;
    }
    
    public int compareTo(CarreraGame game) {
        if (success && game.wasSuccess()) {
            long l = getRaceDuration() - game.getRaceDuration();
            return l < 0 ? -1 : (l == 0 ? 0 : 1);
        }
        return 0;
    }
    
    /**
     * Game control logic of a single race.
     */
    public void run() {
        
        // Aktuelle Einstellungen übermitteln (MIN, MAX, TIME, X-ACCELERATION-T.)
        main.transmitSerial(SerialGameCommands.GAME_CMD_PROFILE);
        String profile = padString(""+main.getMinSpeed(), '0', 2, 1);
        profile += padString(""+main.getMaxSpeed(), '0', 2, 1);
        profile += padString(""+main.getBreakTime(), '0', 4, 1);
        profile += padString(""+main.getXThreshold(), '0', 4, 1);
        char[] c = profile.toCharArray();
        for (int i = 0; i < c.length; i++) {
            main.transmitSerial("" + c[i]);
            try { Thread.sleep(10); } catch (InterruptedException e) {}
        }
        
        // Start counter
        status = STARTING;
        while (startCount > 0 && !isOver()) {
            main.notifyGameListeners(new CarreraGameEvent(this,
                    CarreraGameEvent.EVENT_CLOCK_TICK));
            main.getPlayer().play1();
            try { Thread.sleep(1000); } catch (InterruptedException e) {}
            startCount--;
        }
        
        main.notifyGameListeners(new CarreraGameEvent(this,
                    CarreraGameEvent.EVENT_CLOCK_TICK));
        
        if (isOver()) return;
        
        // Delay a bit (for display of GO!)
        main.getPlayer().play2();
        try { Thread.sleep(250); } catch (InterruptedException e) {}

        if (isOver()) return;
        
        // Start the game
        raceStartTimestamp = System.currentTimeMillis();
        auto.setCurrentLapStartTime(raceStartTimestamp);
        manual.setCurrentLapStartTime(raceStartTimestamp);
        status = STARTED;
        main.transmitSerial(SerialGameCommands.GAME_CMD_START);

        // Main loop, display time
        while (!isOver()) {
            try { Thread.sleep(10); } catch (InterruptedException e) {}
            main.notifyGameListeners(new CarreraGameEvent(this,
                    CarreraGameEvent.EVENT_CLOCK_TICK));
        }
    }
    
    /**
     * Stellt Füllzeichen voran, falls die Länge der Zeichenkette kleiner als die
     * übergebene Länge ist.
     * <p>
     * Die Anzahl der Füllzeichen ergibt sich aus der Differenz von
     * <code>len</code> und der Anzahl der Stellen der Zeichenkette.
     *
     * @param s Die Zeichenkette, welche aufgefüllt werden soll.
     * @param pad Füllzeichen.
     * @param len Länge nach Auffüllung - falls nötig.
     * @param padStyle <code>1</code> falls von Links aufgefüllt werden soll;
     *        <code>0</code> falls von Rechts aufgefüllt werden soll.
     *
     * @return Aufgefüllte Zeichenkette.
     */
    private static String padString(String s, char pad, int len, int padStyle) {
        int currentLength = s.length();
        while (currentLength < len) {
            if (padStyle == 1) {
                s = pad + s;
            } else {
                s = s + pad;
            }
            currentLength++;
        }
        
        return s;
    }

}
